﻿using ComputerShopContracts.BindingModels;
using ComputerShopContracts.ViewModels;
using ComputerShopDataModels.Enums;
using ComputerShopDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerShopDatabaseImplement.Models
{
    internal class Supply : ISupplyModel
    {
        public int Id { get; set; }
        [Required]
        public SupplyStatus Status { get; private set; } = SupplyStatus.Неизвестен;
        [Required]

        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }
        public int ClientId { get; set; }

        public int? ReceivingId { get; set; }
        public virtual EquipmentReceiving Receiving { get; set; }

        private Dictionary<int, IOrderModel>? _supplyOrders =
        null;
        private Dictionary<int, IComponentModel>? _supplyComponents =  null;
        [NotMapped]
        public Dictionary<int, IOrderModel> SupplyOrders
        {
            get
            {
                if (_supplyOrders == null)
                {
                    _supplyOrders = Orders
                    .ToDictionary(recPC => recPC.SupplyId, recPC => recPC.Order as IOrderModel);
                }
                return _supplyOrders;
            }
        }
        [NotMapped]
        public Dictionary<int, IComponentModel> SupplyComponents
        {
            get
            {
                if (_supplyComponents == null)
                {
                    _supplyComponents = Components
                    .ToDictionary(recPC => recPC.SupplyId, recPC => recPC.Component as IComponentModel);
                }
                return _supplyComponents;
            }
        }
        [ForeignKey("SupplyId")]
        public virtual List<SupplyOrder> Orders { get; set; } = new();
        [ForeignKey("SupplyId")]
        public virtual List<ComponentSupply> Components { get; set; } = new();

        public static Supply Create(ComputerShopDatabase context, SupplyBindingModel model)
        {
            return new Supply
            {
                Id = model.Id,
                ClientId = model.ClientId,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ReceivingId = model.ReceivingId,
                Orders = model.SupplyOrders.Select(x => new
                SupplyOrder
                {
                    Order = context.Orders.First(y => y.Id == x.Key),
                }).ToList(),
                Components = model.SupplyComponents.Select(x => new
                ComponentSupply
                {
                    Component = context.Components.First(y => y.Id == x.Key),
                }).ToList()
            };
        }
        public void Update(SupplyBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }
        public SupplyViewModel GetViewModel => new()
        {
            Id = Id,
            Status = Status,           
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            SupplyOrders = SupplyOrders,
            SupplyComponents = SupplyComponents,
            ReceivingId = ReceivingId,
            ClientId = ClientId
        };
        public void UpdateOrders(ComputerShopDatabase context, SupplyBindingModel model)
        {
            var SupplyOrders = context.SupplyOrders.Where(rec =>
            rec.Id == model.Id).ToList();
            if (SupplyOrders != null && SupplyOrders.Count > 0)
            { // удалили те, которых нет в модели
                context.SupplyOrders.RemoveRange(SupplyOrders.Where(rec
                => !model.SupplyOrders.ContainsKey(rec.OrderId)));
                context.SaveChanges();
                // обновили количество у существующих записей
                foreach (var updateOrder in SupplyOrders)
                {
                    model.SupplyOrders.Remove(updateOrder.OrderId);
                }
                context.SaveChanges();
            }
            var supply = context.Supplies.First(x => x.Id == Id);
            foreach (var pc in model.SupplyOrders)
            {
                context.SupplyOrders.Add(new SupplyOrder
                {
                    Supply = supply,
                    Order = context.Orders.First(x => x.Id == pc.Key)
                });
                context.SaveChanges();
            }
            _supplyOrders = null;
        }
    }
}
